******************************************************
*  Program...........: Samples.prg
*  Author............: Lauren Clarke, Randy Pearson
*  Project...........: Parsing Power Tools
*  Created...........: 04/10/2004
*  Copyright.........:
*  Description.......: Code samples for "Parsing Power Tools"
*  Unit Test.........: None
*  Dependencies......: VFP8, RELATION
*  Assumptions.......: SET EXACT OFF
*  Last Change.......: 06/28/2004
*  Change Log........: See end of file
******************************************************
* COMMENTS
*
* Code samples for "Parsing Power Tools Series" by Lauren Clarke and Randy Pearson
* search for the code by Code Listing Number.
* Samples have been gathered into functions for easy calling.
*

* Comment/Uncomment the lines below and execute this prg to see each secton's code samples
CLEAR
* PartOneSamples()
* PartTwoSamples()
PartThreeSamples()


***********************************************************************************************
** PART ONE SAMPLES
***********************************************************************************************

FUNCTION PartOneSamples()

  ? [==== Results: Code Listing 1 ====]
  lcList =  [205.180.85.40;205.140.111.203;206.63.99.226;]+;
    [206.63.99.227;206.63.99.228]
  lnEndPos   = 0
  lnStartPos = 0
  FOR lnK = 1 TO OCCURS([;], lcList) + 1
    lnEndPos = AT( [;], lcList, lnK )
    lnEndPos = IIF( lnEndPos = 0, LEN( lcList ) + 1, lnEndPos )
    lcIP     = SUBSTR( lcList, lnStartPos+1, lnEndPos-lnStartPos-1 )
    ? lcIP   && <= print the IP
    lnStartPos = AT( [;], lcList, lnK )
  ENDFOR


  ? [==== Results: Code Listing 2 ====]

  lcList =  [205.180.85.40;205.140.111.203;206.63.99.226;]+;
    [206.63.99.227;206.63.99.228]

  ALINES(laIPs, lcList, .T., [;] )

  FOR EACH lcIP IN laIPs
    ? m.lcIP
  ENDFOR

  ? [==== Results: Code Listing 3 ====]
  lcList =  [205.180.85.40;205.140.111.203;206.63.99.226;]+;
    [206.63.99.227;206.63.99.228]
  FOR lnK = 1 TO GETWORDCOUNT( lcList, [;] )
    ? GETWORDNUM( lcList, lnK, [;] )
  ENDFOR

  ? [==== Results: Code Listing 4 ====]
  lcList = [205.180.85.40,205.140.111.203;206.63.99.226;206.63.99.227;206.63.99.228;192.168.0.1]
  FOR lnK = 1 TO GETWORDCOUNT( lcList, [;,] )
    ? GETWORDNUM( lcList, lnK, [;,] )
  ENDFOR

  ? [==== Results: Code Listing 5 ====]
*-- normally, we'd extract the colorscheme string from the ini
*   with an API call or another parsing routine
  lcColorscheme=[background,red|foreground,blue|fontcolor,green]

*-- extract the color scheme data from the .ini into an object
  loCScheme = CREATEOBJECT("empty")
  FOR lnK = 1 TO GETWORDCOUNT( lcColorscheme, [|] )
    lcPropValue = GETWORDNUM( lcColorscheme, lnK , [|] )
    lcProperty = GETWORDNUM( lcPropValue,1,[,] )
    lcValue = GETWORDNUM( lcPropValue,2,[,] )
    ADDPROPERTY( loCScheme,lcProperty,lcValue )
  ENDFOR

  WITH loCScheme
    ? .background  && prints "red"
    ? .foreground  && prints "blue"
    ? .fontcolor   && prints "green"
  ENDWITH

  ? [==== Results: Code Listing 6 ====]

*-- see below for UDF
  ? DecimalPlaces(1234.5678)   && prints 4
  ? DecimalPlaces($1234.5678)  && prints 4
  ? DecimalPlaces("1234.5678") && prints 4


  ? [==== Results: Code Listing 7 ====]
TEXT to lcText textmerge noshow pretext 3
    This is a test,
    note how the initial spaces are trimmed
    this is due to the pretext 1 directive above.
    Today is: <<date()>>
ENDTEXT


  ? lcText


  lcText = [The date is <<date()>>]
  ? TEXTMERGE( lcText )
*-- calls to textmerge can be nested
  lcText2 = [The time is <<time()>>, and <<textmerge(lcText)>>]
  ? TEXTMERGE( lcText2 )
*-- or, can be recursed internally
  lcText2 = [The time is <<time()>>, and <<lcText>>]
  ? TEXTMERGE( lcText2 , .T. )


ENDFUNC


*-----------------------------------------------------*
FUNCTION inListString( tcItem, tcList, tcDelimiter, tltrim )

  LOCAL lcTrimmedList
  tcDelimiter = IIF( VARTYPE( m.tcDelimiter ) # [C], [,] , m.tcDelimiter )

  IF m.tltrim
    tcItem = ALLTRIM( m.tcItem )
*-- strip all leading and trailing spaces from list
    lcTrimmedList = []
    FOR lnK = 1 TO GETWORDCOUNT( m.tcList, m.tcDelimiter )
      lcTrimmedList = m.lcTrimmedList + GETWORDNUM( m.tcList, m.lnK, m.tcDelimiter+[ ] )+m.tcDelimiter
    ENDFOR
    m.tcList = m.lcTrimmedList
  ENDIF

  tcItem = m.tcDelimiter + m.tcItem + m.tcDelimiter
  tcList = m.tcDelimiter + m.tcList + m.tcDelimiter

  RETURN m.tcItem $ m.tcList
ENDFUNC



*-----------------------------------------------------*
FUNCTION DecimalPlaces( tnValue ) AS INTEGER
* returns the number of decimal places present in tnValue
  LOCAL lnSetPoint, lnRetVal, lcStringVal
  lcStringVal = PADR( tnValue, 40 )   && max 18 either side of .
  lnRetVal    = 0
  lnSetPoint  = AT( SET("point"), m.lcStringVal )
  IF lnSetPoint > 0
    lnRetVal = LEN( RTRIM( m.lcStringVal ) ) - m.lnSetPoint
  ENDIF
  RETURN m.lnRetVal
ENDFUNC

***********************************************************************************************
** PART TWO SAMPLES
***********************************************************************************************

FUNCTION PartTwoSamples()

  LOCAL lcDemos, lcDecimalToCommaTest, laDemos(1)

TEXT to lcDecimalToCommaTest noshow
    A pair of shoes costs 12.85, an ice-cream cone is 2.60 and a sandwich is 123,412.97.
ENDTEXT

TEXT TO lcDemos NOSHOW pretext 7
    * --- Results: Code Listing from Part Two ---
    validLP("abc123")
    validLP("a1c2b3")

    validLPregExp("abc123")
    validLPregExp("a1c2b3")

    decimalToComma1(lcDecimalToCommaTest)
    decimalToComma2(lcDecimalToCommaTest)
    decimalToComma3(lcDecimalToCommaTest)

    *-- generate three full conversion files for diff
    *strtofile( decimalToComma1(filetostr([ebay2003.html])),[ebay2003_1.html])
    *strtofile( decimalToComma2(filetostr([ebay2003.html])),[ebay2003_2.html])
    *STRTOFILE( decimalToComma3(FILETOSTR([ebay2003.html])),[ebay2003_3.html])

    StrtranRegExp( [The sandwich. cost $5.75. Nice Sandwich.], [(\d{1,3})\.(\d{1,})],[$1,$2] )
    StrtranRegExp( [The sandwich. cost $123,345.75. Nice Sandwich.], [(\d{1,3}?),(\d{1,3}?)],[$1 $2] )

    *-- count words
    OccursRegExp( "\b(\w+)\b",[the then quick quick brown fox fox])
    *-- count repeated words
    OccursRegExp( "\b(\w+)\s\1\b",[the then quick quick brown fox fox])
ENDTEXT

  ALINES( laDemos, lcDemos )

  FOR EACH lcLine IN laDemos
    ? lcLine, [=], IIF(lcLine=[*] OR EMPTY(lcLine),[],EVAL( lcLine ))
  ENDFOR

ENDFUNC


***********************************************************************************************
** PART THREE SAMPLES
***********************************************************************************************

FUNCTION PartThreeSamples()

  LOCAL lcDemos, lcDecimalToCommaTest, laDemos(1)

TEXT TO lcText NOSHOW FLAGS 1
A. 4+7=
B. 7-2=
C. 2^2=
D. 7*(3+1)=
E. MESSAGEBOX('hi')=

ENDTEXT

TEXT TO lcDemos NOSHOW pretext 7
    * --- Results: Code Listing from Part Three ---
    strtranx1([Abracadabra],[a],[*],1,2)
    strtranx1([Abracadabra],[a],[*],1,2,1)
    strtranx1([Abracadabra],[^a|a$],[*],1,2,0)
    strtranx1([Abracadabra],[^a|a$],[*],1,2,1)
    
    * --- Dynamic Substitution examples ---
    strtranx( [The cost, is $123,345.75.], "(\d{1,3})([,\.])(\d{1,3})",[=$1+IIF($2=","," ",",")+$3] )
    strtranx( [6/12/2004 is a weekend, 6/17/2004 is not. ] ,[(\d{1,2}/\d{1,2}/\d{4})],[=IIF(INLIST(DOW(CTOD($1)),1,7),"<strong>"+$1+"</strong>",$1)])
    strtranx( [If text is in CamelCaps, it AB AbA will be linked.] , "(\b\w*[A-Z]+[a-z]+[A-Z]+\w*\b)", [<a href="someurl">$1</a>])
    strtranx( [If text is in CamelCaps, it will be linked.] , "(\b\w*[A-Z]+[a-z]+[A-Z]+\w+\b)", [<a href="someurl">$1</a>])

   *-- note the messagebox is executed
   strtranx2( lcText,"([A-Z]\. )(.+)=",[="$1"+"$2"+"="+TRANSFORM(EVALUATE("$2"))] )
 
   *-- here it is not
   strtranx( lcText,"([A-Z]\. )(\d+)(.+)(\d+)=",[=$1+$2+$3+$4+"="+TRANSFORM(EVALUATE($2+$3+$4))]  )

ENDTEXT

  ALINES( laDemos, lcDemos )

  FOR EACH lcLine IN laDemos
    ? lcLine, [=], IIF(lcLine=[*] OR EMPTY(lcLine),[],EVAL( lcLine ))
  ENDFOR

ENDFUNC








*----------------------------------------------------------
FUNCTION decimalToComma1( tcText )
*-- VFP, with substr
  LOCAL laLines(1), lcLine, lnLines, lcRet, lcTemp

*-- standardize the digits
  lcTemp = CHRTRAN( m.tcText,[0123456789],REPLICATE([#],10))

*-- deal with the decimal places
  FOR lnJ = 1 TO OCCURS( [#.#], m.lcTemp )
    lnPos = AT( [#.#], m.lcTemp, m.lnJ )
    tcText = SUBSTR( m.tcText,1,m.lnPos )+[,]+SUBSTR( m.tcText,m.lnPos+2 )
  ENDFOR

*-- deal with the commas
  FOR lnJ = 1 TO OCCURS( [#,#], m.lcTemp )
    lnPos = AT( [#,#], m.lcTemp, m.lnJ )
    tcText = SUBSTR( m.tcText,1,m.lnPos )+[ ]+SUBSTR( m.tcText,m.lnPos+2 )
  ENDFOR

  RETURN m.tcText

ENDFUNC


*----------------------------------------------------------
FUNCTION decimalToComma2( tcText )

  LOCAL laLines(1), lcLine, lnLines, lcRet, lcTemp

  lnLines = ALINES(laLines, m.tcText, CHR(13) )

  FOR lnK = 1 TO m.lnLines
    lcLine = laLines(m.lnK)

*-- standardize the digits
    lcTemp = CHRTRAN( m.lcLine,[0123456789],REPLICATE([#],10))

*-- deal with the decimals
    FOR lnJ = 1 TO OCCURS( [#.#], m.lcTemp )
      lnPos = AT( [#.#], m.lcTemp, m.lnJ )
      m.lcLine = SUBSTR( m.lcLine,1,m.lnPos )+[,]+SUBSTR( m.lcLine,m.lnPos+2 )
    ENDFOR

*-- deal with the commas
    FOR lnJ = 1 TO OCCURS( [#,#], m.lcTemp )
      lnPos = AT( [#,#], m.lcTemp, m.lnJ )
      m.lcLine = SUBSTR( m.lcLine,1,m.lnPos )+[ ]+SUBSTR( m.lcLine,m.lnPos+2 )
    ENDFOR

    laLines(m.lnK) = m.lcLine
  ENDFOR
  lcRet = []
  FOR lnK = 1 TO m.lnLines
    lcRet= m.lcRet + laLines( m.lnK ) + CHR(13)
  ENDFOR
  RETURN m.lcRet
ENDFUNC

*----------------------------------------------------------
FUNCTION decimalToComma3( tcText )
*-- Regular expressions approach
  LOCAL loRE
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .GLOBAL     = .T.
    .multiline  = .T.
    .ignorecase = .T.
*-- deal with the commas
    .PATTERN    = [(\d{1,3}?),(\d{1,3}?)]
    tcText      = .REPLACE( m.tcText , [$1 $2])
*-- deal with the decimals
    .PATTERN    = [(\d{1,3})\.(\d{1,})]
    RETURN .REPLACE( m.tcText , [$1,$2])
  ENDWITH
ENDFUNC





*----------------------------------------------------------
FUNCTION validLP( tcString )
*[[ paramter checking suppressed for brevity
  tcString = ALLTRIM( tcString )

*-- can't be valid if length # 6
  IF LEN( tcString ) # 6
    RETURN .F.
  ENDIF

*-- standardize all the digits to '#' symbol
  tcString = CHRTRAN( UPPER( tcString ), [1234567890] , REPLICATE([#],10))
*-- standardize all the characters to 'A'
  tcString = CHRTRAN( tcString, [ABCDEFGHIJKLMNOPQRSTUVWXYZ],REPLICATE([A],26) )

*-- check for valid pattern
  RETURN tcString == [AAA###]
ENDFUNC

*----------------------------------------------------------
FUNCTION validLPregExp( tcString )
*-- same as validLP() above, but uses regular expressions
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .PATTERN = "^[A-Za-z]{3}\d{3}$"
    RETURN .test( tcString )
  ENDWITH
ENDFUNC

*----------------------------------------------------------
FUNCTION validLPregx2( tcString, toRE )
*-- same as validLP() above, but uses passed regular expressions
  WITH toRE
    .PATTERN = "^[A-Za-z]{3}\d{3}$"
    RETURN .test( tcString )
  ENDWITH
ENDFUNC

*----------------------------------------------------------
FUNCTION StrtranRegExp( tcSourceString, tcPattern, tcReplace )
  LOCAL loRE
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .PATTERN = tcPattern
    .GLOBAL  = .T.
    .multiline = .T.
    RETURN .REPLACE( tcSourceString , tcReplace )
  ENDWITH
ENDFUNC

*----------------------------------------------------------
FUNCTION OccursRegExp(tcPattern, tcText)
  LOCAL loRE, loMatches, lnResult
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .PATTERN = m.tcPattern
    .GLOBAL  = .T.
    .multiline = .T.
    loMatches = loRE.Execute( m.tcText )
    lnResult  = loMatches.COUNT
    loMatches = NULL
  ENDWITH
  RETURN m.lnResult
ENDFUNC


*----------------------------------------------------------
FUNCTION strtranx(tcSearched, ;
                  tcSearchFor, ;
                  tcReplacement, ;
                  tnStart, tnNumber, ;
                  tnFlag )
                  
  *-- the final version of the UDF
  LOCAL loRE, lcText, lnShift, lcCommand,;
    loMatch, loMatches, lnI, lnK, lcSubMatch,;
    llevaluate, lcMatchDelim, lcReplaceText, lcReplacement,;
    lnStart, lnNumber, loCol, lcKey

  loRE = CREATEOBJECT("vbscript.regexp")

  WITH loRE
    .PATTERN = m.tcSearchFor
    .GLOBAL = .T.
    .multiline = .T.
    .ignorecase = IIF(VARTYPE(m.tnFlag)=[N],m.tnFlag = 1,.F.)
  ENDWITH

  lcReplacement = m.tcReplacement

  *--- are we evaluating?
  IF m.lcReplacement = [=]
    llevaluate       = .T.
    lcReplacement    = SUBSTR( m.lcReplacement, 2 )
  ENDIF

  IF VARTYPE( m.tnStart )=[N]
    lnStart = m.tnStart
  ELSE
    lnStart = 1
  ENDIF

  IF VARTYPE( m.tnNumber) =[N]
    lnNumber = m.tnNumber
  ELSE
    lnNumber = -1
  ENDIF

  IF m.lnStart>1 OR m.lnNumber#-1 OR m.llevaluate

    lcText           = m.tcSearched
    lnShift          = 1
    loMatches        = loRE.execute( m.lcText )
    loCol            = CREATEOBJECT([collection])
    lnNumber = IIF( lnNumber=-1,loMatches.COUNT,MIN(lnNumber,loMatches.COUNT))

    FOR lnK = m.lnStart TO m.lnNumber
      loMatch = loMatches.ITEM(m.lnK-1)  && zero based
      lcCommand = m.lcReplacement
      FOR lnI= 1 TO loMatch.submatches.COUNT
        lcSubMatch = loMatch.submatches(m.lnI-1) && zero based
        IF m.llevaluate
        * "escape" the string we are about to use in an evaluation.
        * it is important to escape due to possible delim chars (like ", ' etc)
        * malicious content, or VFP line-length violations.
          lcKey = ALLTRIM(TRANSFORM(m.lnK)+[_]+TRANSFORM(m.lnI))
          loCol.ADD( m.lcSubMatch, m.lcKey )
          lcSubMatch = [loCol.item(']+m.lcKey+[')]
        ENDIF
        lcCommand = STRTRAN( m.lcCommand, "$" + ALLTRIM( STR( m.lnI ) ) , m.lcSubMatch)
      ENDFOR

      IF m.llevaluate
        TRY
          lcReplaceText = EVALUATE( m.lcCommand )
        CATCH TO loErr
          lcReplaceText="[[ERROR #"+TRANSFORM(loErr.ERRORNO)+[ ]+loErr.MESSAGE+"]]"
        ENDTRY
      ELSE
        lcReplaceText = m.lcCommand
      ENDIF
      lcText  = STUFF( m.lcText, loMatch.FirstIndex + m.lnShift, m.loMatch.LENGTH, m.lcReplaceText )
      lnShift = m.lnShift + LEN( m.lcReplaceText ) - m.loMatch.LENGTH
    ENDFOR
  ELSE
    lcText = loRE.REPLACE( m.tcSearched, m.tcReplacement )
  ENDIF
  RETURN m.lcText
ENDFUNC

*----------------------------------------------------------
FUNCTION strtranx1(tcSearched, tcSearchFor, tcReplacement, tnStart, tnNumber, tnFlag )
*-- does not support VFP expression-based dynamic replacement

  LOCAL loRE, lcText, lnShift, loMatch, loMatches, lnI, lnK, lcSubMatch,;
    lcMatchDelim, lcReplacement, lnStart, lnNumber, loCol, lcKey

  loRE = CREATEOBJECT("vbscript.regexp")

  WITH loRE
    .PATTERN = m.tcSearchFor
    .GLOBAL = .T.
    .multiline = .T.
    *-- not completely adherent to VFP's strtran(), but this
    *   deals with the first two flag settings (0,1)
    .ignorecase = IIF(VARTYPE(m.tnFlag)=[N],m.tnFlag = 1,.F.)
  ENDWITH

  lcReplacement = m.tcReplacement

  IF VARTYPE( m.tnStart )=[N]
    lnStart = m.tnStart
  ELSE
    lnStart = 1
  ENDIF

  IF VARTYPE( m.tnNumber) =[N]
    lnNumber = m.tnNumber
  ELSE
    lnNumber = -1
  ENDIF

  IF m.lnStart>1 OR m.lnNumber#-1
    lcText           = m.tcSearched
    lnShift          = 1
    loMatches        = loRE.execute( m.lcText )
    lnNumber = IIF( lnNumber=-1,loMatches.COUNT,MIN(lnNumber,loMatches.COUNT))
    *-- loop through all the replacement targets
    FOR lnK = m.lnStart TO m.lnNumber
      loMatch = loMatches.ITEM(m.lnK-1)  && zero based
      *-- build the replacement string
      lcReplacement = loRE.REPLACE( loMatch.VALUE, m.tcReplacement )
      *-- stuff the replacement text into the source text
      lcText  = STUFF( m.lcText, ;
        loMatch.FirstIndex + m.lnShift, ;
        m.loMatch.LENGTH, m.lcReplacement)
      *-- keep track of the string position dynamics
      lnShift = m.lnShift + ;
        LEN( m.lcReplacement ) - ;
        m.loMatch.LENGTH
    ENDFOR
  ELSE
    lcText = loRE.REPLACE( m.tcSearched, m.lcReplacement )
  ENDIF
  RETURN m.lcText
ENDFUNC

*----------------------------------------------------------
FUNCTION strtranx2(tcSearched, tcSearchFor, tcReplacement, tnStart, tnNumber, tnFlag )
*--- supports VFP expression-based replacement string, but is not content-safe

  LOCAL loRE, lcText, lnShift, loMatch, loMatches, lnI, lnK, lcSubMatch,;
    lcMatchDelim, lcReplacement, lnStart, lnNumber

  loRE = CREATEOBJECT("vbscript.regexp")

  WITH loRE
    .PATTERN = m.tcSearchFor
    .GLOBAL = .T.
    .multiline = .T.
    .ignorecase = IIF(VARTYPE(m.tnFlag)=[N],m.tnFlag = 1,.F.)
  ENDWITH

  lcReplacement = m.tcReplacement
  llEvaluate = .f.
  *--- are we evaluating?
  IF m.lcReplacement = [=]
    llevaluate       = .T.
    lcReplacement    = SUBSTR( m.lcReplacement, 2 )
  ENDIF

  IF VARTYPE( m.tnStart )=[N]
    lnStart = m.tnStart
  ELSE
    lnStart = 1
  ENDIF

  IF VARTYPE( m.tnNumber) =[N]
    lnNumber = m.tnNumber
  ELSE
    lnNumber = -1
  ENDIF

  IF m.lnStart>1 OR m.lnNumber#-1 OR m.llevaluate
    lcText           = m.tcSearched
    lnShift          = 1
    loMatches        = loRE.execute( m.lcText )

    lnNumber = IIF( lnNumber=-1, ;
                    loMatches.COUNT, ;
                    MIN(lnNumber,loMatches.COUNT))

    *-- loop through all matches
    FOR lnK = m.lnStart TO m.lnNumber
      loMatch = loMatches.ITEM(m.lnK-1)  && zero based
      lcCommand = m.lcReplacement

      *-- build the replacement string
      FOR lnI= 1 TO loMatch.submatches.COUNT
        lcSubMatch = loMatch.submatches(m.lnI-1) && zero based
        lcCommand = STRTRAN(m.lcCommand, "$" + ;
                        ALLTRIM( STR( m.lnI ) ) , m.lcSubMatch)
      ENDFOR
      
      *--- are we evaluating?
      IF m.llevaluate
        TRY
          lcReplacementText = EVALUATE( m.lcCommand )
        CATCH TO loErr
          lcReplacementText="[[ERROR #"+TRANSFORM(loErr.ERRORNO)+[ ]+loErr.MESSAGE+"]]"
        ENDTRY
      ELSE
        lcReplacementText = m.lcReplacement
      ENDIF
     
      *-- stuff the replacement text into the source text
      lcText  = STUFF( m.lcText, ;
        loMatch.FirstIndex + m.lnShift, ;
        m.loMatch.LENGTH, m.lcReplacementText)

      *-- keep track of the string position dynamics
      lnShift = m.lnShift + ;
        LEN( m.lcReplacementText ) - ;
        m.loMatch.LENGTH
    ENDFOR
  ELSE
    lcText = loRE.REPLACE( m.tcSearched, m.lcReplacement )
  ENDIF
  RETURN m.lcText
ENDFUNC
